/******************************************************************************
 * BFProperties.java - Manage user configuration
 * $Id: BFProperties.java,v 1.6 2010/12/29 07:43:20 dick Exp $
 * 
 * BuckoFIBS - Backgammon by BuckoSoft
 * Copyright© 2009,2010 - Dick Balaska - BuckoSoft, Corp.
 * 
 * $Log: BFProperties.java,v $
 * Revision 1.6  2010/12/29 07:43:20  dick
 * Add rightDieType and highDieLeft.
 *
 * Revision 1.5  2010/09/05 04:08:08  dick
 * Version has to be a instantiated class.
 *
 * Revision 1.4  2010/03/03 13:12:21  inim
 * Replaced (c) sign in comment mangled by CVS default encoding back to UTF-8
 *
 * Revision 1.3  2010/03/03 12:19:49  inim
 * Moved source to UTF8 encoding from CP1252 encoding. To this end all source files' (c) message was updated to "Copyright© 2009,2010 - Dick Balaska - BuckoSoft, Corp.". This replaces the (c) sign to UTF8, and adds the new year 2010.
 *
 * Revision 1.2  2010/02/08 09:14:13  dick
 * Add support for autoGreedyBearoff.  When the game turns to a race, enable greedy bearoffs.
 *
 * Revision 1.1  2010/02/04 05:57:53  inim
 * Mavenized project folder layout
 *
 * Revision 1.33  2010/01/29 22:53:40  dick
 * RatingGraphConfig.
 *
 * Revision 1.32  2009/03/12 15:27:34  dick
 * Javadoc.
 *
 * Revision 1.31  2009/02/24 05:20:24  dick
 * add the Ready, Online, Playing property.
 *
 * Revision 1.30  2009/02/22 07:05:17  dick
 * Save the props to the correct place.
 *
 * Revision 1.29  2009/02/21 17:34:33  dick
 * BuckoFIBS.properties becomes BuckoFIBS.cfg.  *.properties is an hsqldb file and  this causes confusion and delay.
 *
 * Revision 1.28  2009/02/14 12:38:13  dick
 * BuckoFIBS is released under the GNU license.
 *
 * Revision 1.27  2009/02/13 07:06:29  dick
 * Return the correct version in the appSignature.
 *
 * Revision 1.26  2009/02/12 06:06:12  dick
 * Add audioCues property.
 *
 * Revision 1.25  2009/02/03 09:43:29  dick
 * Default server is fibs, not buckosoft.
 *
 * Revision 1.24  2009/02/02 08:36:43  dick
 * Add getAppSignature().
 *
 * Revision 1.23  2009/01/28 19:10:37  dick
 * Add hoverHelpers.
 *
 * Revision 1.22  2009/01/18 06:39:20  dick
 * Javadoc.
 *
 * Revision 1.21  2009/01/18 06:37:04  dick
 * Javadoc.
 *
 * Revision 1.20  2009/01/18 06:16:39  dick
 * Javadoc.
 *
 * Revision 1.19  2009/01/18 04:44:50  dick
 * Add showPointNumbers.
 * Reduce the code for boolean gets.
 *
 * Revision 1.18  2009/01/13 18:27:39  dick
 * Handle Ready To Play
 *
 * Revision 1.17  2009/01/12 20:07:14  dick
 * Implement FIBSAttributes as part of the new ClientConnection decoupling.
 *
 * Revision 1.16  2009/01/10 05:32:08  dick
 * Nicer default sizes for those without a properties file (jnlp helper).
 *
 * Revision 1.15  2009/01/10 05:21:02  dick
 * Defaults for some messages is ON.
 *
 * Revision 1.14  2009/01/09 06:57:12  dick
 * Add message routing properties.
 *
 * Revision 1.13  2009/01/07 19:54:21  dick
 * Turn off DEBUG.
 *
 * Revision 1.12  2009/01/07 08:27:37  dick
 * Fix the save timer.
 *
 * Revision 1.11  2009/01/05 07:07:49  dick
 * Add support more multiple profiles.
 *
 * Revision 1.10  2009/01/04 21:31:08  dick
 * Add allowMultiplePersonalities
 *
 * Revision 1.9  2009/01/04 19:57:01  dick
 * Default the username and password to blanks, not null.
 *
 * Revision 1.8  2008/12/13 06:16:14  dick
 * Add NotReady and ShowUnhandledCookies.
 *
 * Revision 1.7  2008/12/12 23:23:02  dick
 * Default values for DispXmit and Recv
 *
 * Revision 1.6  2008/12/11 09:58:08  dick
 * ROT13 the password on the disk.  tee hee.
 *
 * Revision 1.5  2008/12/11 08:45:36  dick
 * Add 3 user prefs, autoConnect, dispXmit and dispRecv.
 *
 * Revision 1.4  2008/12/07 22:52:48  dick
 * Wrap the time in a try block.
 *
 * Revision 1.3  2008/04/01 03:45:48  dick
 * Load/save the window location and position and the splitter's positions.
 *
 * Revision 1.2  2008/03/31 07:07:15  dick
 * Save the properties.
 *
 * Revision 1.1  2008/03/30 05:39:15  dick
 * Manage user configuration.
 */

/* 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * The Original Code is BuckoFIBS, <http://www.buckosoft.com/BuckoFIBS/>.
 * The Initial Developer of the Original Code is Dick Balaska and BuckoSoft, Corp.
 * 
 */
package com.buckosoft.fibs.BuckoFIBS;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.Properties;
import java.util.Timer;
import java.util.TimerTask;

import com.buckosoft.fibs.domain.config.RatingGraphConfig;
import com.buckosoft.fibs.net.FIBSAttributes;

/** Manage user configuration.
 * The client just gets and sets attributes.  This object ensures persistence on its own. 
 * @author Dick Balaska
 * @since 2008/03/30
 * @version $Revision: 1.6 $ <br> $Date: 2010/12/29 07:43:20 $
 * @see <a href="http://cvs.buckosoft.com/Projects/BuckoFIBS/BuckoFIBS/src/main/java/buckosoft/fibs/BuckoFIBS/BFProperties.java">cvs BFProperties.java</a>
 */
public class BFProperties extends Properties implements FIBSAttributes {
	private	final static boolean DEBUG = false;

	private static final long serialVersionUID = 1L;
	final static String	propFile		= "BuckoFIBS.cfg";

	public final static int RIGHT_DIE_PLAYS = 0;
	public final static int RIGHT_DIE_SWAPS = 1;

	final static String	s_selectedProfile	= "selectedProfile";
	final static String s_profileName		= "profileName";
	final static String s_userName			= "userName";
	final static String s_password			= "password";
	final static String s_serverName		= "serverName";
	final static String	s_serverPort		= "serverPort";
	
	final static String s_mHeight			= "mainHeight";
	final static String s_mWidth			= "mainWidth";
	final static String s_mX				= "mainX";
	final static String s_mY				= "mainY";
	final static String s_mTopBottomSplit	= "mainTBSplit";
	final static String s_mTopLRSplit		= "mainTLRSplit";
	final static String s_mBotLRSplit		= "mainBLRSplit";
	final static String s_showPointNumbers	= "showPointNumbers";
	final static String s_autoConnect		= "autoConnect";
	final static String s_dispXmit			= "displayXmit";
	final static String s_dispRecv			= "displayRecv";
	final static String	s_allowMultiPerson	= "multiplePerson";
	final static String	s_hoverHelpers		= "hoverHelpers";
	final static String	s_audioCues			= "audioCues";
	final static String s_ratingGraphType	= "ratingGraphType";
	final static String s_ratingGraphAmt	= "ratingGraphAmt";

	final static String s_msgIgnoreShouts	= "msgIgnoreShouts";
	final static String s_msgShowGameMoves	= "msgShowGameMoves";
	final static String s_msgOtherMatchInfo	= "msgOtherMatchInfo";
	final static String s_msgLoginsAndOuts	= "msgLoginsAndOuts";
	final static String s_fibsReadyToPlay	= "fibsReadyToPlay";
	final static String s_rop				= "rop";
	final static String s_autoGreedyBearOff	= "autoGreedyBearOff";
	final static String s_rightDieType		= "rightDieType";
	final static String s_highDieLeft		= "highDieLeft";

	final static String s_DEBUG_NotReady	= "NotReady";
	final static String s_DEBUG_ShowUnhandledCookies = "ShowUnhandledCookies";
	final static String s_DEBUG_StdoutNetworkMessages = "stdoutNetMessages";

	/** If there are no props saved, then this is our first time running BuckoFIBS */
	private  boolean newbie = false;

	private	RatingGraphConfig	ratingGraphConfig = new RatingGraphConfig();
	
	/** Create a new BFProperties. <br>
	 * File reading and parsing is done in this constructor.
	 */
	public BFProperties() {
		try {
		    FileInputStream fip = new FileInputStream(System.getProperty("user.home") 
		    		+ File.separator + "BuckoFIBS" + File.separator + propFile);
		    load(fip);
		}
		catch (Exception e) {
		    System.out.println("Exception while loading props:" + e);
		    newbie = true;
		}
		
		if (getProperty(s_selectedProfile) == null)
			setProperty(s_selectedProfile, "0");

		if (getProperty(s_serverName + 0) == null)
			setProperty(s_serverName + 0, "fibs.com");
		if (getProperty(s_serverPort + 0) == null)
			setProperty(s_serverPort + 0, "4321");
		if (getProperty(s_userName + 0) == null)
			setProperty(s_userName + 0, "");
		if (getProperty(s_password + 0) == null)
			setProperty(s_password + 0, "");

		if (getProperty(s_mWidth) == null)
			setProperty(s_mWidth, "936");
		if (getProperty(s_mHeight) == null)
			setProperty(s_mHeight, "840");
		if (getProperty(s_mX) == null)
			setProperty(s_mX, "10");
		if (getProperty(s_mY) == null)
			setProperty(s_mY, "10");
		if (getProperty(s_mTopBottomSplit) == null)
			setProperty(s_mTopBottomSplit, "482");
		if (getProperty(s_mTopLRSplit) == null)
			setProperty(s_mTopLRSplit, "532");
		if (getProperty(s_mBotLRSplit) == null)
			setProperty(s_mBotLRSplit, "123");
		if (getProperty(s_autoConnect) == null)
			setProperty(s_autoConnect, "0");
		if (getProperty(s_autoGreedyBearOff) == null)
			setProperty(s_autoGreedyBearOff, "1");
		if (getProperty(s_rightDieType) == null)
			setProperty(s_rightDieType, "" + RIGHT_DIE_PLAYS);
		if (getProperty(s_highDieLeft) == null)
			setProperty(s_highDieLeft, "1");
		
		if (getProperty(s_showPointNumbers) == null)
			setProperty(s_showPointNumbers, "1");
		if (getProperty(s_dispXmit) == null)
			setProperty(s_dispXmit, "0");
		if (getProperty(s_dispRecv) == null)
			setProperty(s_dispRecv, "0");
		if (getProperty(s_allowMultiPerson) == null)
			setProperty(s_allowMultiPerson, "0");
		if (getProperty(s_hoverHelpers) == null)
			setProperty(s_hoverHelpers, "1");
		if (getProperty(s_audioCues) == null)
			setProperty(s_audioCues, "1");
		if (getProperty(s_rop) == null)
			setProperty(s_rop, "111");
		if (getProperty(s_ratingGraphType) == null)
			setProperty(s_ratingGraphType, "0");
		if (getProperty(s_ratingGraphAmt) == null)
			setProperty(s_ratingGraphAmt, "50");

		if (getProperty(s_msgIgnoreShouts) == null)
			setProperty(s_msgIgnoreShouts, "0");
		if (getProperty(s_msgShowGameMoves) == null)
			setProperty(s_msgShowGameMoves, "1");
		if (getProperty(s_msgOtherMatchInfo) == null)
			setProperty(s_msgOtherMatchInfo, "1");
		if (getProperty(s_msgLoginsAndOuts) == null)
			setProperty(s_msgLoginsAndOuts, "1");
		
		if (getProperty(s_fibsReadyToPlay) == null)
			setProperty(s_fibsReadyToPlay, "1");

		if (getProperty(s_DEBUG_NotReady) == null)
			setProperty(s_DEBUG_NotReady, "0");
		if (getProperty(s_DEBUG_ShowUnhandledCookies) == null)
			setProperty(s_DEBUG_ShowUnhandledCookies, "0");
		if (getProperty(s_DEBUG_StdoutNetworkMessages) == null)
			setProperty(s_DEBUG_StdoutNetworkMessages, "0");
	}

	private	Timer	myTimer = null;
	private	TimerTask	myTimerTask = new MyTimerTask(this);
	private	void	propsChanged() {
		if (myTimer != null) {
			return;
		}
		myTimer = new Timer();
		try {
			myTimerTask = new MyTimerTask(this);
			myTimer.schedule(myTimerTask, 1000);
		} catch (IllegalStateException e) {
			e.printStackTrace();
		}
	}

	private class MyTimerTask extends TimerTask {
		private	BFProperties props;

		MyTimerTask(BFProperties props) {
			super();
			this.props = props;
		}
		/* (non-Javadoc)
		 * @see java.util.TimerTask#run()
		 */
		@Override
		public void run() {
			this.props.save();
			this.props.myTimer.cancel();
			this.props.myTimer = null;
		}
	}
	
	/* (non-Javadoc)
	 * @see com.buckosoft.fibs.net.FIBSAttributes#getAppSignature()
	 */
	@Override
	public String getAppSignature() {
		Version v = new Version();
		return("BuckoFIBS-" + v.getVersion());
	}

	/** Is this user a newbie?
	 * @return the newbie
	 */
	public boolean isNewbie() {
		return newbie;
	}

	/** Get the id of the currently selected profile.
	 * @return The selected profile id
	 */
	public int getSelectedProfile() {
		return(Integer.parseInt(getProperty(s_selectedProfile)));
	}

	/** Set the id of the currently selected profile.
	 * @param profileId The id of the profile that the user selected.
	 */
	public void setSelectedProfile(int profileId) {
		if (getSelectedProfile() != profileId)
			propsChanged();
		setProperty(s_selectedProfile, "" + profileId);
	}

	/** Get the number of profiles that we are keeping track of
	 * @return The number of profiles
	 */
	public int getProfileCount() {
		int profileCount = 0;
		while (true) {
			if (getProperty(s_profileName + profileCount) == null)
				return(profileCount);
			profileCount++;
		}
	}

	/** Get the name of this profile (not neccessarily the login name)
	 * @param profileId The profile to query
	 * @return The name of the profile
	 */ 
	public String	getProfileName(int profileId) {
		return(getProperty(s_profileName + profileId));
	}

	/** Set the name of this profile (not neccessarily the login name)
	 * @param profileId The profile to set
	 * @param s The name of the profile
	 */
	public void setProfileName(int profileId, String s) {
		if (!s.equals(getProfileName(profileId)))
			propsChanged();
		setProperty(s_profileName + profileId, s);
	}

	/** Get the user's login name for the currently selected profile.
	 * @return The user name
	 */
	public String getUserName() {
		return(this.getUserName(this.getSelectedProfile()));
	}

	/** Get the user's login name for this profile
	 * @param profileId The profile id to query
	 * @return The user name or an empty string if not found.
	 */ 
	public String	getUserName(int profileId) {
		String s = getProperty(s_userName + profileId);
		if (s == null)
			s = "";
		return(s);
	}
	
	/** Set the user's login name for this profile.
	 * @param profileId The profile id to query.
	 * @param s The username
	 */ 
	public void setUserName(int profileId, String s) {
		if (!s.equals(getUserName(profileId)))
			propsChanged();
		setProperty(s_userName + profileId, s);
	}

	/** Get the clear text password of the user in the currently selected profile
	 * @return The password
	 */
	public String getUserPassword() {
		return(this.getPassword(this.getSelectedProfile()));
	}

	/** Get the clear text password of the user in this profile.
	 * @param profileId The profile id to query.
	 * @return The password
	 */
	public String	getPassword(int profileId) {
		String t = getProperty(s_password + profileId);
		if (t == null)
			return("");
		t = ROT13.doConvert(t);
		
		if (DEBUG)
			System.out.println("get password = " + t);
		return(t);
	}
	
	/** Set the clear text password of the user for this profile.
	 * @param profileId The profile id to query.
	 * @param s The clear text password
	 */
	public void setPassword(int profileId, String s) {
		if (!s.equals(getPassword(profileId)))
			propsChanged();
		String t = ROT13.doConvert(s);
		setProperty(s_password + profileId, t);
		if (DEBUG)
			System.out.println("set password = " + getProperty(s_password));
	}

	/** Get the name of the server to connect to for the currently selected profile
	 * @return The server name 
	 */
	public String getServerName() {
		return(this.getServerName(this.getSelectedProfile()));
	}

	/** get the name of the server we connect to
	 * @return The name of the server
	 */
	public String	getServerName(int profileId) {
		String s = getProperty(s_serverName + profileId);
		if (s == null)
			s = "lbd.buckosoft.com";
		return(s);
	}

	/** Set the name of the server that we connect to in this profile
	 * @param profileId The profile id to set
	 * @param s The server name.
	 */
	public void setServerName(int profileId, String s) {
		if (!s.equals(getServerName(profileId)))
			propsChanged();
		setProperty(s_serverName + profileId, s);
	}

	/** Get the IP port number to connect to for the currently selected profile.
	 * @return The port (typically 4321)
	 */
	public int getServerPort() {
		return(this.getServerPort(this.getSelectedProfile()));
	}

	/** Get the IP port number to connect to for this profile
	 * @param profileId The profile id to query
	 * @return The port (typically 4321)
	 */ 
	public int getServerPort(int profileId) {
		String s = getProperty(s_serverPort + profileId);
		if (s == null)
			return(4321);
		return(Integer.parseInt(s));
	}
	
	/** Set the IP port number to connect to for this profile
	 * @param profileId The profile id to query
	 * @param port The port number (if it's not 4321, then uh oh)
	 */
	public void setServerPort(int profileId, int port) {
		if (port != getServerPort(profileId))
			propsChanged();
		setProperty(s_serverPort + profileId, new Integer(port).toString());
	}

	/** Get the width for the MainDialog
	 * @return The width in pixels
	 */
	public int		getMainWidth() {
		String s = getProperty(s_mWidth);
		return(Integer.parseInt(s));
	}
	
	/** Set the width of the MainDialog
	 * @param width The width in pixels
	 */
	public void setMainWidth(int width) {
		if (width != getMainWidth())
			propsChanged();
		setProperty(s_mWidth, new Integer(width).toString());
	}

	public int getRightDieType() {
		String s = getProperty(s_rightDieType);
		return(Integer.parseInt(s));
	}
	
	public void setRightDieType(int rightDieType) {
		if (rightDieType != getRightDieType())
			propsChanged();
		setProperty(s_rightDieType, "" + rightDieType);
	}

	public boolean isHighDieLeft() {
		String s = getProperty(s_highDieLeft);
		return(Integer.parseInt(s) == 1 ? true : false);
	}
	
	public void setHighDieLeft(boolean highDieLeft) {
		if (highDieLeft != isHighDieLeft())
			propsChanged();
		setProperty(s_highDieLeft, highDieLeft ? "1" : "0");
	}

	/** Get the height for the MainDialog
	 * @return The height in pixels
	 */
	public int		getMainHeight() {
		String s = getProperty(s_mHeight);
		return(Integer.parseInt(s));
	}

	/** Set the height for the MainDialog
	 * @param height The height in pixels.
	 */ 
	public void setMainHeight(int height) {
		if (height != getMainHeight())
			propsChanged();
		setProperty(s_mHeight, new Integer(height).toString());
	}

	/** Get the X position for the MainDialog. <br>
	 * This is where to place the dialog on the screen during creation
	 * @return The X position
	 */
	public int		getMainX() {
		String s = getProperty(s_mX);
		return(Integer.parseInt(s));
	}
	
	/** Set the X position of the MainDialog on the screen.
	 * @param x The X position
	 */
	public void setMainX(int x) {
		if (x != getMainX())
			propsChanged();
		setProperty(s_mX, new Integer(x).toString());
	}

	/** Get the Y position for the MainDialog. <br>
	 * This is where to place the dialog on the screen during creation
	 * @return The Y position
	 */
	public int		getMainY() {
		String s = getProperty(s_mY);
		return(Integer.parseInt(s));
	}

	/** Set the Y position of the MainDialog on the screen.
	 * @param y The Y position
	 */
	public void setMainY(int y) {
		if (y != getMainY())
			propsChanged();
		setProperty(s_mY, new Integer(y).toString());
	}

	/** Get the position of the split for the main top/bottom split in the MainDialog
	 * @return The position to set the splitter at.
	 */ 
	public int		getMainTopBottomSplit() {
		String s = getProperty(s_mTopBottomSplit);
		return(Integer.parseInt(s));
	}

	/** Set the position of the split for the main top/bottom split in the MainDialog
	 * @param sp The position to set the splitter at.
	 */ 
	public void setMainTopBottomSplit(int sp) {
		if (sp != getMainTopBottomSplit())
			propsChanged();
		setProperty(s_mTopBottomSplit, new Integer(sp).toString());
	}

	/** Get the position of the Left/Right split in the top half of the MainDialog
	 * @return The position of the split
	 */ 
	public int		getMainTopLRSplit() {
		String s = getProperty(s_mTopLRSplit);
		return(Integer.parseInt(s));
	}

	/** Set the position of the Left/Right split in the top half of the MainDialog
	 * @param sp The position of the split
	 */ 
	public void setMainTopLRSplit(int sp) {
		if (sp != getMainTopLRSplit())
			propsChanged();
		setProperty(s_mTopLRSplit, new Integer(sp).toString());
	}
	
	/** Get the position of the Left/Right split in the bottom half of the MainDialog
	 * @return The position of the split
	 */ 
	public int		getMainBotLRSplit() {
		String s = getProperty(s_mBotLRSplit);
		return(Integer.parseInt(s));
	}

	/** Set the position of the Left/Right split in the bottom half of the MainDialog
	 * @param sp The position of the split
	 */ 
	public void setMainBotLRSplit(int sp) {
		if (sp != getMainBotLRSplit())
			propsChanged();
		setProperty(s_mBotLRSplit, new Integer(sp).toString());
	}
	
	/** Should we display the numbers on the points on the board?
	 * @return true == yes
	 */
	public boolean isShowPointNumbers() {
		String s = getProperty(s_showPointNumbers);
		return(s.equals("1"));
	}
	
	/** Set whether to display the numbers on the points on the board.
	 * @param b true == yes
	 */
	public void setShowPointNumbers(boolean b) {
		if (b != isShowPointNumbers())
			propsChanged();
		setProperty(s_showPointNumbers, b ? "1" : "0");
	}

	/** Should we automatically connect to the server when the app starts up?
	 * @return true == yes
	 */
	public boolean isAutoConnect() {
		String s = getProperty(s_autoConnect);
		return(s.equals("1"));
	}
	
	/** Set the auto connect value
	 * @param b true == yes
	 */
	public void setAutoConnect(boolean b) {
		if (b != isAutoConnect())
			propsChanged();
		setProperty(s_autoConnect, b ? "1" : "0");
	}

	/** Should we automatically turn on Greedy Bear Offs when the game turns to a race?
	 * @return true == yes
	 */
	public boolean isAutoGreedyBearOff() {
		String s = getProperty(s_autoGreedyBearOff);
		return(s.equals("1"));
	}
	
	/** Set the auto Greedy Bear Off
	 * @param b true == yes
	 */
	public void setAutoGreedyBearOff(boolean b) {
		if (b != isAutoGreedyBearOff())
			propsChanged();
		setProperty(s_autoGreedyBearOff, b ? "1" : "0");
	}

	/** Should we display messages that we transmit to the fibs server?
	 * @return true == display them
	 */
	public boolean isDisplayXmit() {
		String s = getProperty(s_dispXmit);
		return(s.equals("1"));
	}

	/** Set whether to display messages that we transmit to the fibs server.
	 * @param b true == display them
	 */
	public void setDisplayXmit(boolean b) {
		if (b != isDisplayXmit())
			propsChanged();
		setProperty(s_dispXmit, b ? "1" : "0");
	}

	/** OBSOLETE::
	 * @return OBSOLETE
	 */
	public boolean isDisplayRecv() {
		String s = getProperty(s_dispRecv);
		return(s.equals("1"));
	}

	/** OBSOLETE:
	 * @param b
	 */
	public void setDisplayRecv(boolean b) {
		if (b != isDisplayRecv())
			propsChanged();
		setProperty(s_dispRecv, b ? "1" : "0");
	}

	/** Should we display received network messages on stdout?
	 * @return true == yes
	 */ 
	public boolean isStdoutNetworkMessages() {
		String s = getProperty(s_DEBUG_StdoutNetworkMessages);
		return(s.equals("1"));
	}

	/** Set whether to display received network messages on stdout.
	 * @param b true == display 'em
	 */
	public void setStdoutNetworkMessages(boolean b) {
		if (b != isStdoutNetworkMessages())
			propsChanged();
		setProperty(s_DEBUG_StdoutNetworkMessages, b ? "1" : "0");
	}

	/** Should we allow the user to have multiple personalities?
	 * The default is no, this is really only a development feature so that i
	 * can connect to fibs.com or buckosoft.com
	 * @return true == yes
	 */
	public boolean isAllowMultiplePersonalities() {
		String s = getProperty(s_allowMultiPerson);
		return(s.equals("1"));
	}

	/** Set the value for allow multiple personalities
	 * @param b true == yes
	 */
	public void setAllowMultiplePersonalities(boolean b) {
		if (b != isAllowMultiplePersonalities())
			propsChanged();
		setProperty(s_allowMultiPerson, b ? "1" : "0");
	}

	/** Should we show the hover helpers during a player's move
	 * The default is yes, max eye candy is on.
	 * @return true == yes
	 */
	public boolean isHoverHelpers() {
		String s = getProperty(s_hoverHelpers);
		return(s.equals("1"));
	}

	/** Should we show the hover helpers during a player's move
	 * @param b true == yes
	 */
	public void setHoverHelpers(boolean b) {
		if (b != isHoverHelpers())
			propsChanged();
		setProperty(s_hoverHelpers, b ? "1" : "0");
	}

	/** Should we play the beeps and boops?
	 * @return true = make noise
	 */
	public boolean isAudioCues() {
		String s = getProperty(s_audioCues);
		return(s.equals("1"));		
	}

	/** Set whether to play the audio cues
	 * @param b false = don't make noise
	 */
	public void setAudioCues(boolean b) {
		if (b != isAudioCues())
			propsChanged();
		setProperty(s_audioCues, b ? "1" : "0");
	}

	/** Should we ignore people shouting? (global chat messages)
	 * @return true == ignore them
	 */
	public boolean isMsgIgnoreShouts() {
		String s = getProperty(s_msgIgnoreShouts);
		return(s.equals("1"));
	}
	
	/** Set whether to ignore shouts or not
	 * @param b true == ignore them
	 */
	public void setMsgIgnoreShouts(boolean b) {
		if (b != isMsgIgnoreShouts())
			propsChanged();
		setProperty(s_msgIgnoreShouts, b ? "1" : "0");
	}

	/** Should we display game move messages (in its own window)
	 * @return true == yup
	 */
	public boolean isMsgShowGameMoves() {
		String s = getProperty(s_msgShowGameMoves);
		return(s.equals("1"));
	}

	/** Set whether to display game move messages
	 * @param b true == display them
	 */
	public void setMsgShowGameMoves(boolean b) {
		if (b != isMsgShowGameMoves())
			propsChanged();
		setProperty(s_msgShowGameMoves, b ? "1" : "0");
	}

	/** Should we display messages about other matches?
	 * @return true == display them
	 */
	public boolean isMsgOtherMatchInfo() {
		String s = getProperty(s_msgOtherMatchInfo);
		return(s.equals("1"));
	}
	
	/** Set whether to display messages about other messages
	 * @param b true == display them
	 */
	public void setMsgOtherMatchInfo(boolean b) {
		if (b != isMsgOtherMatchInfo())
			propsChanged();
		setProperty(s_msgOtherMatchInfo, b ? "1" : "0");
	}

	/** Should we display messages about other people logging in and out?
	 * @return true == display them
	 */
	public boolean isMsgLoginsAndOuts() {
		String s = getProperty(s_msgLoginsAndOuts);
		return(s.equals("1"));
	}
	
	/** Set whether to display messages about other people logging in and out
	 * @param b true == display them
	 */
	public void setMsgLoginsAndOuts(boolean b) {
		if (b != isMsgLoginsAndOuts())
			propsChanged();
		setProperty(s_msgLoginsAndOuts, b ? "1" : "0");
	}

	/** Has the user marked himself as ready to play?
	 * @return true == ja
	 */
	public boolean isFibsReadyToPlay() {
		String s = getProperty(s_fibsReadyToPlay);
		return(s.equals("1"));
	}
	
	/** Set whether we are ready to play
	 * @param b true == yes
	 */
	public void setFibsReadyToPlay(boolean b) {
		if (b != isFibsReadyToPlay())
			propsChanged();
		setProperty(s_fibsReadyToPlay, b ? "1" : "0");
	}

	/** Set the state of the Ready, Online, Playing triad.
	 * @param rop rop[0] = the Ready state, rop[1] = the Online state, rop[2] = the Playing state.
	 */
	public void setROP(boolean[] rop) {
		if (rop != getROP())
			propsChanged();
		String s = (rop[0] ? "1" : "0")
				 + (rop[1] ? "1" : "0")
				 + (rop[2] ? "1" : "0");
		setProperty(s_rop, s);
		
	}
	
	/** Get the state of the Ready, Online, Playing triad.
	 * @return b[0] = the Ready state, b[1] = the Online state, b[2] = the Playing state.
	 */
	public boolean[] getROP() {
		boolean[] b = new boolean[3];
		String s = getProperty(s_rop);
		b[0] = s.charAt(0) == '1' ? true : false;
		b[1] = s.charAt(1) == '1' ? true : false;
		b[2] = s.charAt(2) == '1' ? true : false;
		return(b);
	}

	public RatingGraphConfig	getRatingGraphConfig() {
		String s = getProperty(s_ratingGraphType);
		ratingGraphConfig.setTypeAsInt(Integer.parseInt(s));
		s = getProperty(s_ratingGraphAmt);
		ratingGraphConfig.setMatchCount(Integer.parseInt(s));
		return(ratingGraphConfig);
	}

	public void setRatingGraphConfig(RatingGraphConfig ratingGraphConfig) {
		this.ratingGraphConfig = ratingGraphConfig;
		setProperty(s_ratingGraphType, "" + this.ratingGraphConfig.getTypeAsInt());
		setProperty(s_ratingGraphAmt, "" + this.ratingGraphConfig.getMatchCount());		
		propsChanged();
	}

	/** Should we display unhandled cookies as an error?
	 * @return true == yes
	 */
	public boolean isDEBUG_ShowUnhandledCookies() {
		String s = getProperty(s_DEBUG_ShowUnhandledCookies);
		return(s.equals("1"));
	}

	/** Set whether we should display unhandled cookies
	 * @param b true == oui
	 */
	public void setDEBUG_ShowUnhandledCookies(boolean b) {
		if (b != isDEBUG_ShowUnhandledCookies())
			propsChanged();
		setProperty(s_DEBUG_ShowUnhandledCookies, b ? "1" : "0");
	}

	///////////////////////////////////////////////////////////////////////////
	static class ROT13 {

	    static private int abyte = 0;

	    public static String doConvert(String in) {

	        StringBuffer tempReturn = new StringBuffer();


	        for (int i=0; i<in.length(); i++) {

	            abyte = in.charAt(i);
	            int cap = abyte & 32;
	            abyte &= ~cap;
	            abyte = ( (abyte >= 'A') && (abyte <= 'Z') ? ((abyte - 'A' + 13) % 26 + 'A') : abyte) | cap;
	            tempReturn.append((char)abyte);
	        }
	        return tempReturn.toString();
	        
	    }
	}
	
	/** Save the properties file
	 */
	private void save() {
		if (DEBUG)
			System.out.println("Saving properties");
		try {
			FileOutputStream fos = new FileOutputStream(System.getProperty("user.home")
					+ File.separator + "BuckoFIBS" + File.separator + propFile);
			store(fos, "BuckoFIBS properties");
		} catch (Exception e) {
			System.out.println("Exception while saving props:" + e);
		}
	}


}
