/******************************************************************************
 * ProfileSelectorPanel.java - A little panel the selects a profile, or creates a new one.
 * $Id: ProfileSelectorPanel.java,v 1.3 2010/03/03 13:12:21 inim Exp $
 * 
 * BuckoFIBS - Backgammon by BuckoSoft
 * Copyright© 2009,2010 - Dick Balaska - BuckoSoft, Corp.
 * 
 * $Log: ProfileSelectorPanel.java,v $
 * Revision 1.3  2010/03/03 13:12:21  inim
 * Replaced (c) sign in comment mangled by CVS default encoding back to UTF-8
 *
 * Revision 1.2  2010/03/03 12:19:49  inim
 * Moved source to UTF8 encoding from CP1252 encoding. To this end all source files' (c) message was updated to "Copyright© 2009,2010 - Dick Balaska - BuckoSoft, Corp.". This replaces the (c) sign to UTF8, and adds the new year 2010.
 *
 * Revision 1.1  2010/02/04 05:57:53  inim
 * Mavenized project folder layout
 *
 * Revision 1.5  2009/01/28 18:15:10  dick
 * Prettier cvs link in the javadoc.
 *
 * Revision 1.4  2009/01/09 07:18:55  dick
 * Don't fire events during construction.  (Causes us to 'select' profile 0 before ever reading the data).
 *
 * Revision 1.3  2009/01/07 08:36:34  dick
 * Make it functional.
 *
 * Revision 1.2  2009/01/06 08:11:24  dick
 * Pass the owning dialog in the constructor.
 * Select 0 if really -1.
 *
 * Revision 1.1  2009/01/05 07:16:35  dick
 * A little panel the selects a profile, or creates a new one.
 *
 */
package com.buckosoft.fibs.BuckoFIBS.gui.account;

import java.awt.Font;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

import com.buckosoft.fibs.BuckoFIBS.BFProperties;

/** A panel to manage which profile is selected. 
 * @author Dick Balaska
 * @since 2009/01/04
 * @version $Revision: 1.3 $ <br> $Date: 2010/03/03 13:12:21 $
 * @see <a href="http://cvs.buckosoft.com/Projects/BuckoFIBS/BuckoFIBS/src/com/buckosoft/fibs/BuckoFIBS/gui/account/ServerPortSelectorPanel.java">cvs ServerPortSelectorPanel.java</a>
 */
public class ProfileSelectorPanel extends JPanel {
	private static final long serialVersionUID = 1L;
	private	JDialog	owningDialog = null;
	private JComboBox jComboBoxProfileName = null;
	private JButton jButtonNewProfile = null;
	private	BFProperties properties;
	private	ProfileSelectorNotifier profileSelectorNotifier;
	private	boolean initializing = true;	// don't let events fire prematurely

	/**
	 * This method initializes 
	 * 
	 */
	public ProfileSelectorPanel() {
		super();
		initialize();
		initializing = false;
	}

	public ProfileSelectorPanel(JDialog owner, ProfileSelectorNotifier profileSelectorNotifier, BFProperties properties) {
		super();
		this.owningDialog = owner;
		this.profileSelectorNotifier = profileSelectorNotifier;
		this.properties = properties;
		initialize();
		setupProfiles();
		initializing = false;
	}

	public int getSelectedProfile() {
		if (!this.isVisible())
			return(0);
		if (this.jComboBoxProfileName.getSelectedIndex() == -1)
			return(0);
		return(this.jComboBoxProfileName.getSelectedIndex());
	}

	private void setupProfiles() {
		int profileCount = this.properties.getProfileCount();
		if (profileCount == 0) {
			String s = "Profile 0";
			this.jComboBoxProfileName.addItem(s);
			this.properties.setProfileName(0, s);
			this.properties.setSelectedProfile(0);
			return;
		}
		for (int i=0; i<profileCount; i++) {
			String s = this.properties.getProfileName(i);
			if (s != null && s.length() > 0) {
				this.jComboBoxProfileName.addItem(s);
			} else {
				break;
			}
		}
		this.jComboBoxProfileName.setSelectedIndex(this.properties.getSelectedProfile());
	}

	private void profileChanged(java.awt.event.ItemEvent e) {
		if (initializing)
			return;
		if (e.getStateChange() == java.awt.event.ItemEvent.SELECTED) {
			this.properties.setSelectedProfile(this.jComboBoxProfileName.getSelectedIndex());
			profileSelectorNotifier.profileChanged();
		}
	}

	/**
	 * This method initializes this
	 * 
	 */
	private void initialize() {
        this.add(getJComboBoxProfileName(), null);
        this.add(getJButtonNewProfile(), null);
	}

	/**
	 * This method initializes jComboBoxProfileName	
	 * 	
	 * @return javax.swing.JComboBox	
	 */
	private JComboBox getJComboBoxProfileName() {
		if (jComboBoxProfileName == null) {
			jComboBoxProfileName = new JComboBox();
			jComboBoxProfileName.addItemListener(new java.awt.event.ItemListener() {
				public void itemStateChanged(java.awt.event.ItemEvent e) {
					profileChanged(e);
				}
			});
		}
		return jComboBoxProfileName;
	}

	/**
	 * This method initializes jButtonNewProfile	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getJButtonNewProfile() {
		if (jButtonNewProfile == null) {
			jButtonNewProfile = new JButton();
			jButtonNewProfile.setText("New");
			jButtonNewProfile.setFont(new Font("Dialog", Font.PLAIN, 10));
			jButtonNewProfile.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					showNewProfileDialog();
				}
			});
		}
		return jButtonNewProfile;
	}

	/** Show the dialog and return the name of the new Profile
	 * @return The name of the new Profile, or null
	 */
	private	void showNewProfileDialog() {
		//newProfileNameDialog = new NewProfileNameDialog(owningDialog);
		int profileId = this.properties.getProfileCount();
		String defaultName = "Profile " + profileId;
		String s = (String)JOptionPane.showInputDialog(
				this.owningDialog.getComponent(0),
                "Type a name for the new Profile",
                "New Profile Name",
                JOptionPane.PLAIN_MESSAGE,
                null,
                null,
                defaultName);
		if (s == null || s.length() == 0)
			return;
		this.properties.setProfileName(profileId, s);
		this.properties.setUserName(profileId, "");
		this.properties.setPassword(profileId, "");
		this.jComboBoxProfileName.addItem(s);
		this.jComboBoxProfileName.setSelectedIndex(profileId);
	}
}
