/******************************************************************************
 * PlayerListPopupMenu.java - The popup menu for the PlayerList
 * $Id: PlayerListPopupMenu.java,v 1.5 2011/01/01 06:12:31 dick Exp $
 * 
 * BuckoFIBS - Backgammon by BuckoSoft
 * Copyright© 2009,2010 - Dick Balaska - BuckoSoft, Corp.
 * 
 * $Log: PlayerListPopupMenu.java,v $
 * Revision 1.5  2011/01/01 06:12:31  dick
 * Don't need to keep GroupManager local.
 *
 * Revision 1.4  2011/01/01 00:19:08  dick
 * Build the groups for the submenu.
 *
 * Revision 1.3  2010/03/03 13:12:21  inim
 * Replaced (c) sign in comment mangled by CVS default encoding back to UTF-8
 *
 * Revision 1.2  2010/03/03 12:19:49  inim
 * Moved source to UTF8 encoding from CP1252 encoding. To this end all source files' (c) message was updated to "Copyright© 2009,2010 - Dick Balaska - BuckoSoft, Corp.". This replaces the (c) sign to UTF8, and adds the new year 2010.
 *
 * Revision 1.1  2010/02/04 05:57:53  inim
 * Mavenized project folder layout
 *
 * Revision 1.5  2010/01/24 06:21:36  dick
 * Don't update the playerList if the popupmenu is open, it gets confused.
 *
 * Revision 1.4  2010/01/23 06:26:04  dick
 * Add the RepBot menu.
 *
 * Revision 1.3  2009/02/14 15:46:36  dick
 * BuckoFIBS is released under the GNU license.
 *
 * Revision 1.2  2009/02/01 21:28:25  dick
 * Handle the InvitePopupSubmenu.
 *
 * Revision 1.1  2009/01/28 19:39:44  dick
 * package com.buckosoft.fibs.gui.playerList becomes com.buckosoft.fibs.BuckoFIBS.gui.playerList.
 *
 * Revision 1.3  2009/01/24 17:08:03  dick
 * Javadoc.
 *
 * Revision 1.2  2008/12/12 14:46:36  dick
 * Enable the Watch command.
 *
 * Revision 1.1  2008/04/07 08:08:25  dick
 * The popup menu for the PlayerList.
 */

/* 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * The Original Code is BuckoFIBS, <http://www.buckosoft.com/BuckoFIBS/>.
 * The Initial Developer of the Original Code is Dick Balaska and BuckoSoft, Corp.
 * 
 */
package com.buckosoft.fibs.BuckoFIBS.gui.playerList;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;

import com.buckosoft.fibs.BuckoFIBS.GroupManager;
import com.buckosoft.fibs.BuckoFIBS.gui.playerList.group.GroupPopupSubmenu;
import com.buckosoft.fibs.domain.Player;

/** The popup menu for the PlayerList.
 * @author Dick Balaska
 * @since 2008/04/07
 * @version $Revision: 1.5 $ <br> $Date: 2011/01/01 06:12:31 $
 * @see <a href="http://cvs.buckosoft.com/Projects/BuckoFIBS/BuckoFIBS/src/main/java/com/buckosoft/fibs/BuckoFIBS/gui/playerList/PlayerListPopupMenu.java">cvs PlayerListPopupMenu.java</a>
 */
public class PlayerListPopupMenu extends JPopupMenu implements ActionListener {
	private	final static boolean DEBUG = false;
	private static final long serialVersionUID = 1L;

	private JMenuItem watchMenuItem = null;
	private JMenuItem lookMenuItem = null;
	private JMenuItem unwatchMenuItem = null;
	private JMenuItem whoisMenuItem = null;
	private JMenuItem viewProfileMenuItem = null;
	private JMenuItem updateMenuItem = null;
	private	InvitePopupSubmenu	invitePopupSubmenu = null;
	private RepBotPopupSubmenu	repbotPopupSubmenu = null;
	private GroupPopupSubmenu	groupPopupSubmenu = null;

	private	ActionListener	menuHandler = null;  //  @jve:decl-index=0:

	private	Player		player;
	
	public final static String s_setVisible = "setVisible";
	/**
	 * This method initializes 
	 * 
	 */
	public PlayerListPopupMenu() {
		super();
		initialize();
	}

	/**
	 * This method initializes this
	 * 
	 */
	private void initialize() {
		this.add(getInvitePopupSubmenu());
		this.add(getGroupPopupSubmenu());
        this.addSeparator();
        this.add(getLookMenuItem());
        this.add(getWatchMenuItem());
        this.add(getUnwatchMenuItem());
        this.addSeparator();
		this.add(getRepBotPopupSubmenu());
        this.add(getWhoisMenuItem());
        this.add(getViewProfileMenuItem());
        this.add(getUpdateMenuItem());
			
	}

	/** Set our reference to the object that will receive menu events
	 * @param c The ActionListener object
	 */ 
	public	void setMenuHandler(ActionListener c) {
		this.menuHandler = c;
		this.getInvitePopupSubmenu().setMenuHandler(c);
		this.getRepBotPopupSubmenu().setMenuHandler(c);
	}

	public void setGroupManager(GroupManager groupManager) {
		this.getGroupPopupSubmenu().setGroupManager(groupManager);
	}
	
	public void setPlayer(Player player) {
		this.player = player;
	}

	/* (non-Javadoc)
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent e) {
		menuHandler.actionPerformed(e);
	}

	/**
	 * This method initializes watchMenuItem	
	 * 	
	 * @return javax.swing.JMenuItem	
	 */
	private JMenuItem getWatchMenuItem() {
		if (watchMenuItem == null) {
			watchMenuItem = new JMenuItem();
			watchMenuItem.setText("Watch");
			watchMenuItem.addActionListener(this);
		}
		return watchMenuItem;
	}

	/**
	 * This method initializes lookMenuItem	
	 * 	
	 * @return javax.swing.JMenuItem	
	 */
	private JMenuItem getLookMenuItem() {
		if (lookMenuItem == null) {
			lookMenuItem = new JMenuItem();
			lookMenuItem.setText("Look");
			lookMenuItem.setEnabled(false);
			lookMenuItem.addActionListener(this);
		}
		return lookMenuItem;
	}

	/**
	 * This method initializes unwatchMenuItem	
	 * 	
	 * @return javax.swing.JMenuItem	
	 */
	private JMenuItem getUnwatchMenuItem() {
		if (unwatchMenuItem == null) {
			unwatchMenuItem = new JMenuItem();
			unwatchMenuItem.setText("Unwatch");
			unwatchMenuItem.setEnabled(false);
			unwatchMenuItem.addActionListener(this);
		}
		return unwatchMenuItem;
	}

	/**
	 * This method initializes whoisMenuItem	
	 * 	
	 * @return javax.swing.JMenuItem	
	 */
	private JMenuItem getWhoisMenuItem() {
		if (whoisMenuItem == null) {
			whoisMenuItem = new JMenuItem();
			whoisMenuItem.setText("Whois");
			whoisMenuItem.setEnabled(false);
			whoisMenuItem.addActionListener(this);
		}
		return whoisMenuItem;
	}

	/**
	 * This method initializes viewProfileMenuItem	
	 * 	
	 * @return javax.swing.JMenuItem	
	 */
	private JMenuItem getViewProfileMenuItem() {
		if (viewProfileMenuItem == null) {
			viewProfileMenuItem = new JMenuItem();
			viewProfileMenuItem.setText("View Profile");
			viewProfileMenuItem.setEnabled(false);
			viewProfileMenuItem.addActionListener(this);
		}
		return viewProfileMenuItem;
	}

	/**
	 * This method initializes updateMenuItem	
	 * 	
	 * @return javax.swing.JMenuItem	
	 */
	private JMenuItem getUpdateMenuItem() {
		if (updateMenuItem == null) {
			updateMenuItem = new JMenuItem();
			updateMenuItem.setText("Update");
			updateMenuItem.setEnabled(true);
			updateMenuItem.addActionListener(this);
		}
		return updateMenuItem;
	}

	protected	InvitePopupSubmenu	getInvitePopupSubmenu() {
		if (invitePopupSubmenu == null) {
			invitePopupSubmenu = new InvitePopupSubmenu();
			invitePopupSubmenu.addActionListener(this);
		}
		return(invitePopupSubmenu);
	}

	protected	RepBotPopupSubmenu	getRepBotPopupSubmenu() {
		if (repbotPopupSubmenu == null) {
			repbotPopupSubmenu = new RepBotPopupSubmenu();
			repbotPopupSubmenu.addActionListener(this);
		}
		return(repbotPopupSubmenu);
	}

	protected GroupPopupSubmenu getGroupPopupSubmenu() {
		if (groupPopupSubmenu == null) {
			groupPopupSubmenu = new GroupPopupSubmenu();
		}
		return(groupPopupSubmenu);
	}

	/* (non-Javadoc)
	 * @see javax.swing.JPopupMenu#setVisible(boolean)
	 */
	@Override
	public void setVisible(boolean arg0) {
		super.setVisible(arg0);
		if (DEBUG)
			System.out.println("PlayerListPopupMenu: setVisible(" + arg0 + ")");
		ActionEvent ae = new ActionEvent(this, 0, s_setVisible, arg0 ? 1 : 0);
		this.actionPerformed(ae);
	}

	public void rebuildGroups() {
		this.groupPopupSubmenu.repopulate(player);
	}
}
