/******************************************************************************
 * GroupSelectPane.java - Display the pane the allows the user to select active groups
 * $Id: GroupSelectPane.java,v 1.3 2011/01/01 20:41:33 dick Exp $
 * 
 * BuckoFIBS - Backgammon by BuckoSoft
 * Copyright© 2010 - Dick Balaska - BuckoSoft, Corp.
 * 
 * $Log: GroupSelectPane.java,v $
 * Revision 1.3  2011/01/01 20:41:33  dick
 * We have to use buttons instead of labels in order to figure out what we clicked on.
 * I don't like the way they look, but they work.
 *
 * Revision 1.2  2011/01/01 06:23:51  dick
 * Enable/disable groups on selecting All.
 *
 * Revision 1.1  2010/12/31 05:34:51  dick
 * Display the pane the allows the user to select active groups.
 *
 */

/* 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * The Original Code is BuckoFIBS, <http://www.buckosoft.com/BuckoFIBS/>.
 * The Initial Developer of the Original Code is Dick Balaska and BuckoSoft, Corp.
 * 
 */
package com.buckosoft.fibs.BuckoFIBS.gui.playerList.group;

import java.awt.BorderLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ItemEvent;
import java.util.LinkedList;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import com.buckosoft.fibs.BuckoFIBS.GroupManager;
import com.buckosoft.fibs.BuckoFIBS.gui.MainDialog;
import com.buckosoft.fibs.domain.GroupOfPlayers;

/** Display the pane the allows the user to select active groups.
 * @author Dick Balaska
 * @since 2010/12/30
 * @version $Revision: 1.3 $ <br> $Date: 2011/01/01 20:41:33 $
 * @see <a href="http://cvs.buckosoft.com/Projects/BuckoFIBS/BuckoFIBS/src/main/java/com/buckosoft/fibs/BuckoFIBS/gui/playerList/PlayerListPane.java">cvs PlayerListPane.java</a>
 */
public class GroupSelectPane extends JPanel {
	private static final boolean DEBUG = false;
	private static final long serialVersionUID = 1L;
	private JPanel jPanelMainPane = null;
	private JCheckBox jCheckBoxAll = null;
//	private JLabel jLabelAll = null;
//	private JLabel jLabelAddNewGroup = null;
	private JScrollPane scrollPane = null;  //  @jve:decl-index=0:visual-constraint="17,265"
	private	GroupManager	groupManager = null;
	private	MainDialog		mainDialog = null;
	
	private LinkedList<CheckWithName> checkWithNameList = new LinkedList<CheckWithName>();
	
	class CheckWithName {
		int			id;
		JCheckBox	checkBox;
		JButton		label;
		String		name;
	}

	/**
	 * This is the default constructor
	 */
	public GroupSelectPane() {
		super();
		initialize();
	}

/*	public GroupSelectPane(JComponent parent) {
		super(parent);
		initialize();
	}
*/
	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		System.out.println("GroupSelectPane:<init>");
		this.setLayout(new BorderLayout());
		this.setSize(300, 200);
		this.add(getScrollPane());
	}

	private JScrollPane getScrollPane() {
		if (scrollPane == null) {
			scrollPane = new JScrollPane();
			scrollPane.setViewportView(getJPanelMainPane());
		}
		return(scrollPane);
	}

	/**
	 * This method initializes jPanelMainPane	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getJPanelMainPane() {
		if (jPanelMainPane == null) {
			jPanelMainPane = new JPanel();
			jPanelMainPane.setLayout(new GridBagLayout());
		}
		return jPanelMainPane;
	}

	public void setGroupManager(GroupManager groupManager) {
		this.groupManager = groupManager;
		this.groupManager.setGroupSelectPane(this);
	}

	public void setMainDialog(MainDialog mainDialog) {
		this.mainDialog = mainDialog;
	}
	/**
	 * This method initializes jCheckBoxAll	
	 * 	
	 * @return javax.swing.JCheckBox	
	 */
	private JCheckBox getJCheckBoxAll() {
		if (jCheckBoxAll == null) {
			jCheckBoxAll = new JCheckBox();
		}
		return jCheckBoxAll;
	}

	public void repopulate() {
		//this.remove(this.getScrollPane());
		jPanelMainPane = null;
		scrollPane.setViewportView(getJPanelMainPane());

		//scrollPane = null;
	}

	public void populatePane() {
		this.groupManager.populatePane();
	}

	public void addGroup(GroupOfPlayers groupOfPlayers) {
		GridBagConstraints gridBagConstraintsR = new GridBagConstraints();
		gridBagConstraintsR.insets = new Insets(0, 0, 0, 1);
		gridBagConstraintsR.gridx = 1;
		//gridBagConstraintsR.anchor = GridBagConstraints.WEST;
		gridBagConstraintsR.gridy = this.checkWithNameList.size();
		GridBagConstraints gridBagConstraintsL = new GridBagConstraints();
		gridBagConstraintsL.gridx = 0;
		gridBagConstraintsL.gridy = this.checkWithNameList.size();
		gridBagConstraintsL.insets = new Insets(0, 1, 0, 0);

		String s = groupOfPlayers.getGroupName();
		JCheckBox	cb;
		if (s.equals("All"))
			cb = getJCheckBoxAll();
		else {
			cb = new JCheckBox();
			cb.setName(s);
		}
		cb.addItemListener(new java.awt.event.ItemListener() {
			public void itemStateChanged(java.awt.event.ItemEvent e) {
				if (DEBUG)
					System.out.println("Group toggled: ");
				groupChecked(e);
			}
		});
		cb.getModel().setSelected(groupOfPlayers.isActive());
		jPanelMainPane.add(cb, gridBagConstraintsL);
		JButton l = null;
		if (s.equals("All")) {
			JLabel jl = new JLabel();
			jl.setText(s);
			jPanelMainPane.add(jl, gridBagConstraintsR);
		} else {
			l = new JButton();
			l.setBorder(null);
			l.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					System.out.println("actionPerformed()"); // TODO Auto-generated Event stub actionPerformed()
					onEditGroup((JButton)e.getSource());
				}
			});
			gridBagConstraintsR.fill = GridBagConstraints.HORIZONTAL;
			if (groupOfPlayers.isIgnore())
				l.setText("!" + s);
			else
				l.setText(s);
			jPanelMainPane.add(l, gridBagConstraintsR);
		}
		CheckWithName cwn = new CheckWithName();
		cwn.id = groupOfPlayers.getId();
		cwn.checkBox = cb;
		cwn.name = s;
		cwn.label = l;
		checkWithNameList.add(cwn);
		
	}
	public void populateFinish() {
		GridBagConstraints gridBagConstraints = new GridBagConstraints();
		gridBagConstraints.gridwidth = 2;
		gridBagConstraints.gridx = 0;
		//gridBagConstraints.gridy = 1;
		gridBagConstraints.insets = new Insets(10, 0, 0, 0);
		gridBagConstraints.anchor = GridBagConstraints.SOUTH;
		JLabel l = new JLabel();
		l.setText("Add New Group...");
		l.addMouseListener(new java.awt.event.MouseAdapter() {
			public void mouseClicked(java.awt.event.MouseEvent e) {
				handleAddNewGroupButton();
			}
		});
		jPanelMainPane.add(l, gridBagConstraints);
		updateDisabledItems();
	}
	public void savePaneToList() {
		for (CheckWithName cwn : checkWithNameList) {
			this.groupManager.updateGroupOfPlayers(cwn.id, cwn.name, cwn.checkBox.getModel().isSelected());
		}
	}
	
	private void handleAddNewGroupButton() {
		EditGroupDialog egd = new EditGroupDialog(mainDialog, this);
		egd.setVisible(true);
	}
	
	public void addNewGroupFromUser(GroupOfPlayers groupOfPlayers) {
		this.groupManager.addNewGroupFromUser(groupOfPlayers);
	}

	public void editGroupFromUser(GroupOfPlayers groupOfPlayers) {
		this.groupManager.editGroupFromUser(groupOfPlayers);
	}

	/** Here when the "All" checkbox is toggled
	 * @param checked true = checked
	 */
	private void groupChecked(ItemEvent e) {
		updateDisabledItems();
		for (CheckWithName cwn : checkWithNameList) {
			if (cwn.checkBox == e.getItem()) {
				this.groupManager.updateGroupOfPlayers(cwn.id, cwn.name, cwn.checkBox.getModel().isSelected());
			}
		}
	}

	/** Depending on certain events, items get enabled and disabled.  Process them here.
	 */
	private void updateDisabledItems() {
		boolean allMode = this.getJCheckBoxAll().getModel().isSelected();
		for (CheckWithName cwn : checkWithNameList) {
			if (cwn.name.equals("All"))
				continue;
			cwn.checkBox.setEnabled(!allMode);
			cwn.label.setEnabled(!allMode);
		}
	}
	
	private void onEditGroup(JButton e) {
		for (CheckWithName cwn : checkWithNameList) {
			if (cwn.name.equals("All"))
				continue;
			if (cwn.label == e) {
				EditGroupDialog egd = new EditGroupDialog(mainDialog, this);
				egd.setGroupOfPlayers(this.groupManager.getGroupOfPlayers(cwn.id));
				egd.setVisible(true);
				return;
			}
		}
	}
}
